==> cargo-registry-0.1.0.crate: cargo-registry-0.1.0/src/api.rs <==
//! The registry HTTP API.
//!
//! This implements the specification from [here][api_spec].
//!
//! To simplify the code and avoid over-abstracting, operations on the
//! database and the filesystem are mainly implemented in this module
//! instead of in the [database](crate::database) and
//! [storage](crate::storage) modules respectively.
//!
//! [api_spec]: https://doc.rust-lang.org/cargo/reference/registries.html#web-api

use crate::{auth::User, database::Database, index, storage::Storage};
use rocket::{fs::NamedFile, response::Responder, serde::json::Json, Data, Route, State};
use serde::{Deserialize, Serialize};
use std::{
    collections::HashMap,
    sync::{Arc, Mutex},
};
use tokio::io::{AsyncReadExt as _, AsyncWriteExt as _};
use tracing::{error, info};

/// A simple type alias to add the crate index to a request.
type Index = Arc<Mutex<crate::index::Index>>;

type Result<T> = std::result::Result<T, Error>;

#[derive(Debug, thiserror::Error)]
enum Error {
    #[error("IO")]
    IO(#[from] std::io::Error),

    #[error("Index: {0}")]
    Index(#[from] index::Error),

    #[error("Json: {0}")]
    Json(#[from] serde_json::Error),

    #[error("SQL: {0}")]
    Sql(#[from] sqlx::Error),

    #[error("User does not own the crate")]
    NotOwnedByUser,
}

impl<'r> Responder<'r, 'static> for Error {
    fn respond_to(self, request: &'r rocket::Request<'_>) -> rocket::response::Result<'static> {
        use rocket::http::Status;

        let status = match self {
            _ => Status::BadRequest,
        };

        let base = Json(serde_json::json!({
            "error": status.reason(),
            "reason": self.to_string(),
        }))
        .respond_to(request)?;

        rocket::Response::build().merge(base).status(status).ok()
    }
}

/// Start here. Return the list of routes that make up the API.
///
/// For `cargo` to work with the API, these should most likely be
/// mounted at `/api/v1`:
///
/// ```
/// rocket::ignite().mount("/api/v1", routes())
/// ```
///
/// See also [`root_routes`].
pub fn routes() -> Vec<Route> {
    rocket::routes![
        download,
        publish,
        yank,
        unyank,
        owners_list,
        owners_put,
        owners_remove,
        search,
    ]
}

/// Additional routes that should be mounted at `/`.
pub fn root_routes() -> Vec<Route> {
    rocket::routes![me]
}

// Download

#[rocket::get("/crates/<crate_name>/<version>/download")]
async fn download(
    crate_name: String,
    version: String,
    storage: &State<Storage>,
) -> Result<NamedFile> {
    let path = storage
        .path
        .join(format!("{}-{}.crate", crate_name, version));
    NamedFile::open(path).await.map_err(Error::IO)
}

// Publish

#[derive(Deserialize)]
struct PublishCrateBody {
    pub name: String,
    pub vers: String,
    pub deps: Vec<Dependency>,
    pub features: HashMap<String, Vec<String>>,
    pub authors: Vec<String>,
    pub description: Option<String>,
    pub homepage: Option<String>,
    pub readme: Option<String>,
    pub readme_file: Option<String>,
    pub keywords: Vec<String>,
    pub categories: Vec<String>,
    pub license: Option<String>,
    pub license_file: Option<String>,
    pub repository: Option<String>,
    #[serde(default)]
    pub badges: HashMap<String, Badge>,
    pub links: Option<String>,
}

#[derive(Deserialize)]
struct Dependency {
    /// Name of the dependency.
    ///
    /// If the dependency is renamed from the original package name,
    /// this is the original name. The new package name is stored in
    /// the `explicit_name_in_toml` field.
    pub name: String,

    /// The semver requirement for this dependency.
    pub version_req: String,

    /// Array of features (as strings) enabled for this dependency.
    pub features: Vec<String>,

    /// Boolean of whether or not this is an optional dependency.
    pub optional: bool,

    /// Boolean of whether or not default features are enabled.
    pub default_features: bool,

    /// The target platform for the dependency.
    /// null if not a target dependency.
    /// Otherwise, a string such as "cfg(windows)".
    pub target: Option<String>,

    /// The dependency kind.
    /// "dev", "build", or "normal".
    pub kind: DependencyKind,

    /// The URL of the index of the registry where this dependency is
    /// from as a string. If not specified or null, it is assumed the
    /// dependency is in the current registry.
    pub registry: Option<String>,

    /// If the dependency is renamed, this is a string of the new
    /// package name. If not specified or null, this dependency is not
    /// renamed.
    pub explicit_name_in_toml: Option<String>,
}

impl From<Dependency> for index::Dependency {
    fn from(dep: Dependency) -> Self {
        index::Dependency {
            name: dep.name,
            req: dep.version_req,
            features: dep.features,
            optional: dep.optional,
            default_features: dep.default_features,
            target: dep.target,
            kind: dep.kind.into(),
            registry: dep.registry,
            package: dep.explicit_name_in_toml,
        }
    }
}

#[derive(Deserialize)]
#[serde(rename_all = "snake_case")]
enum DependencyKind {
    Dev,
    Build,
    Normal,
}

impl From<DependencyKind> for index::DependencyKind {
    fn from(kind: DependencyKind) -> Self {
        match kind {
            DependencyKind::Dev => index::DependencyKind::Dev,
            DependencyKind::Build => index::DependencyKind::Build,
            DependencyKind::Normal => index::DependencyKind::Normal,
        }
    }
}

#[derive(Deserialize)]
#[serde(transparent)]
struct Badge(HashMap<String, String>);

#[derive(Serialize)]
struct PublishResponse {
    invalid_categories: Vec<String>,
    invalid_badges: Vec<String>,
    other: Vec<String>,
}

#[rocket::put("/crates/new", data = "<body>")]
async fn publish(
    user: User,
    body: Data<'_>,
    database: &State<Database>,
    index: &State<Index>,
    storage: &State<Storage>,
) -> Result<Json<PublishResponse>> {
    // Read crate metadata.
    let mut stream = body.open(rocket::data::ByteUnit::Gigabyte(1));
    let metadata_len = stream.read_u32_le().await? as usize;
    let mut metadata_buffer = vec![0; metadata_len];
    stream.read_exact(&mut metadata_buffer).await?;
    let metadata: PublishCrateBody = serde_json::from_slice(&metadata_buffer)?;

    // Either check that the user is an owner of the crate, or insert
    // a new crate into the database.
    let crate_id = {
        let mut trans = database.pool.begin().await?;

        let row = sqlx::query!("select crate_id from crate where name = $1", metadata.name)
            .fetch_optional(&mut trans)
            .await?;

        let crate_id = match row {
            // The crate exists; ensure that the user is an owner.
            Some(row) => {
                user_owns_crate(&user, &metadata.name, &database.pool).await?;
                row.crate_id
            }
            // The crate does not exist; create it and make the user
            // an owner.
            None => {
                let crate_id = sqlx::query!(
                    "insert into crate (name) values ($1) returning crate_id",
                    metadata.name
                )
                .fetch_one(&mut trans)
                .await?
                .crate_id;

                sqlx::query!(
                    "insert into crate_owner (crate_id, account_id) values ($1, $2)",
                    crate_id,
                    user.account_id as i32
                )
                .execute(&mut trans)
                .await?;

                crate_id
            }
        };

        trans.commit().await?;

        crate_id
    };

    // Read .crate file.
    let crate_len = stream.read_u32_le().await? as usize;
    let mut crate_buffer = vec![0; crate_len];
    stream.read_exact(&mut crate_buffer).await?;
    let cksum = {
        use sha2::Digest;
        let digest = sha2::Sha256::digest(&crate_buffer);
        format!("{:x}", digest)
    };

    let crate_path = storage
        .path
        .join(format!("{}-{}.crate", metadata.name, metadata.vers));
    info!(?crate_path, "opening crate file");
    let mut file = tokio::fs::OpenOptions::new()
        .write(true)
        .create_new(true)
        .open(&crate_path)
        .await?;
    info!(?crate_path, "writing crate file");
    file.write_all(&crate_buffer).await?;

    sqlx::query!(
        "update crate set description = $1 where crate_id = $2",
        metadata.description,
        crate_id
    )
    .execute(&database.pool)
    .await?;

    sqlx::query!(
        "insert into crate_version (crate_id, version) values ($1, $2)",
        crate_id,
        metadata.vers
    )
    .execute(&database.pool)
    .await?;

    let deps = metadata
        .deps
        .into_iter()
        .map(index::Dependency::from)
        .collect();
    let crate_info = index::Crate {
        name: metadata.name,
        vers: metadata.vers,
        deps,
        cksum,
        features: HashMap::new(),
        yanked: false,
        links: None,
    };

    tokio::task::block_in_place(|| {
        info!("updating index");
        let index = index.lock().unwrap();
        index.add_crate(crate_info)?;
        Result::Ok(())
    })?;

    let response = PublishResponse {
        invalid_categories: vec![],
        invalid_badges: vec![],
        other: vec![],
    };
    Ok(Json(response))
}

// Yank

#[derive(Serialize)]
struct YankResponse {
    /// Indicates the delete succeeded, always true.
    ok: bool,
}

#[rocket::delete("/crates/<crate_name>/<version>/yank")]
async fn yank(
    user: User,
    crate_name: String,
    version: String,
    database: &State<Database>,
    index: &State<Index>,
) -> Result<Json<YankResponse>> {
    user_owns_crate(&user, &crate_name, &database.pool).await?;

    tokio::task::block_in_place(|| {
        let index = index.lock().unwrap();
        index.yank(&crate_name, &version, true)?;
        Result::Ok(())
    })?;

    let response = YankResponse { ok: true };
    Ok(Json(response))
}

// Unyank

#[rocket::put("/crates/<crate_name>/<version>/unyank")]
async fn unyank(
    user: User,
    crate_name: String,
    version: String,
    database: &State<Database>,
    index: &State<Index>,
) -> Result<Json<YankResponse>> {
    user_owns_crate(&user, &crate_name, &database.pool).await?;

    tokio::task::block_in_place(|| {
        let index = index.lock().unwrap();
        index.yank(&crate_name, &version, false)?;
        Result::Ok(())
    })?;

    let response = YankResponse { ok: true };
    Ok(Json(response))
}

// Owners: List

#[derive(Serialize)]
struct OwnersListResponse {
    users: Vec<Owner>,
}

#[derive(Serialize)]
struct Owner {
    id: u32,
    login: String,
    name: Option<String>,
}

#[rocket::get("/crates/<crate_name>/owners")]
async fn owners_list(
    user: User,
    crate_name: String,
    database: &State<Database>,
) -> Result<Json<OwnersListResponse>> {
    user_owns_crate(&user, &crate_name, &database.pool).await?;

    let owners = sqlx::query!("select account_id, username from crate join crate_owner using (crate_id) join account using (account_id) where crate.name = $1", crate_name).fetch_all(&database.pool).await?;
    let users = owners
        .into_iter()
        .map(|owner| Owner {
            id: owner.account_id as u32,
            login: owner.username,
            name: None,
        })
        .collect();

    let response = OwnersListResponse { users };
    Ok(Json(response))
}

// Owners: Add

#[derive(Deserialize)]
struct OwnersPutBody {
    /// Array of `login` strings of owners to add.
    users: Vec<String>,
}

#[derive(Serialize)]
struct OwnersPutResponse {
    ok: bool,
    msg: &'static str,
}

#[rocket::put("/crates/<crate_name>/owners", data = "<body>")]
async fn owners_put(
    user: User,
    crate_name: String,
    body: Json<OwnersPutBody>,
    database: &State<Database>,
) -> Result<Json<OwnersPutResponse>> {
    user_owns_crate(&user, &crate_name, &database.pool).await?;

    let mut trans = database.pool.begin().await?;
    let crate_id = sqlx::query!("select crate_id from crate where name = $1", crate_name)
        .fetch_one(&mut trans)
        .await?
        .crate_id;
    for user in &body.users {
        let account_id = sqlx::query!("select account_id from account where username = $1", user)
            .fetch_one(&mut trans)
            .await?
            .account_id;
        sqlx::query!(
            "insert into crate_owner (crate_id, account_id) values ($1, $2)",
            crate_id,
            account_id,
        )
        .execute(&mut trans)
        .await?;
    }
    trans.commit().await?;

    let response = OwnersPutResponse {
        ok: true,
        msg: "success",
    };
    Ok(Json(response))
}

// Owners: Remove

#[derive(Deserialize)]
struct OwnersDeleteBody {
    /// Array of `login` strings of owners to remove.
    users: Vec<String>,
}

#[derive(Serialize)]
struct OwnersDeleteResponse {
    ok: bool,
    msg: &'static str,
}

#[rocket::delete("/crates/<crate_name>/owners", data = "<body>")]
async fn owners_remove(
    user: User,
    crate_name: String,
    body: Json<OwnersDeleteBody>,
    database: &State<Database>,
) -> Result<Json<OwnersDeleteResponse>> {
    user_owns_crate(&user, &crate_name, &database.pool).await?;

    let mut trans = database.pool.begin().await?;
    let crate_id = sqlx::query!("select crate_id from crate where name = $1", crate_name)
        .fetch_one(&mut trans)
        .await?
        .crate_id;
    for user in &body.users {
        sqlx::query!("delete from crate_owner where crate_id = $1 and account_id = (select account_id from account where username = $2)", crate_id, user).execute(&mut trans).await?;
    }
    trans.commit().await?;

    let response = OwnersDeleteResponse {
        ok: true,
        msg: "success",
    };
    Ok(Json(response))
}

// Search

#[derive(Serialize)]
struct SearchResponse {
    /// Array of results.
    crates: Vec<SearchResultCrate>,
    meta: SearchResponseMeta,
}

#[derive(Serialize)]
struct SearchResultCrate {
    /// Name of the crate.
    name: String,
    /// The highest version available.
    // TODO: semver
    max_version: String,
    /// Textual description of the crate.
    description: String,
}

#[derive(Serialize)]
struct SearchResponseMeta {
    /// Total number of results available on the server.
    total: usize,
}

#[rocket::get("/crates?<q>&<per_page>")]
async fn search(
    q: Option<String>,
    per_page: Option<i64>,
    database: &State<Database>,
) -> Result<Json<SearchResponse>> {
    let crates = sqlx::query!("select name, max(version) as max_version, description from crate left join crate_version using (crate_id) where name ~ $1 group by crate_id limit $2", q.unwrap_or_default(), per_page)
        .fetch_all(&database.pool)
        .await?;
    let crates = crates
        .into_iter()
        .map(|row| SearchResultCrate {
            name: row.name,
            max_version: row.max_version.unwrap_or_else(|| "0.0.0".into()),
            description: row.description.unwrap_or_default(),
        })
        .collect();

    let response = SearchResponse {
        crates,
        meta: SearchResponseMeta { total: 1 },
    };
    Ok(Json(response))
}

// Login

// User info

#[derive(Serialize)]
struct LoggedInUser {
    username: String,
}

#[rocket::get("/me")]
async fn me(user: User, database: &State<Database>) -> Result<Json<LoggedInUser>> {
    let row = sqlx::query!(
        "select username from account where account_id = $1",
        user.account_id as i32
    )
    .fetch_one(&database.pool)
    .await?;

    let response = LoggedInUser {
        username: row.username,
    };
    Ok(Json(response))
}

// Helpers

/// Check whether the user is an owner of the given crate.
async fn user_owns_crate(user: &User, crate_name: &str, pool: &sqlx::PgPool) -> Result<()> {
    let row = sqlx::query!(
        "select 1 as x from crate join crate_owner using (crate_id) where crate.name = $1 and crate_owner.account_id = $2",
        crate_name, user.account_id as i32
    ).fetch_optional(pool).await?;

    match row {
        Some(_) => Ok(()),
        None => Err(Error::NotOwnedByUser),
    }
}

==> cargo-registry-0.2.2.crate: cargo-registry-0.2.2/src/api.rs <==
//! The registry HTTP API.
//!
//! This implements the specification from [here][api_spec].
//!
//! To simplify the code and avoid over-abstracting, operations on the
//! database and the filesystem are mainly implemented in this module
//! instead of in the [database](crate::database) and
//! [storage](crate::storage) modules respectively.
//!
//! [api_spec]: https://doc.rust-lang.org/cargo/reference/registries.html#web-api

use crate::{auth::User, database::Database, index, storage::Storage};
use axum::{
    body::StreamBody,
    extract::{Extension, Path, Query},
    response::{IntoResponse, Response},
    routing::{delete, get, put},
    Json,
};
use futures::prelude::*;
use http::StatusCode;
use serde::{Deserialize, Serialize};
use std::{
    collections::HashMap,
    sync::{Arc, Mutex},
};
use tokio::io::{AsyncReadExt as _, AsyncWriteExt as _};
use tokio_util::io::{ReaderStream, StreamReader};
use tracing::{error, info};

/// A simple type alias to add the crate index to a request.
type Index = Arc<Mutex<crate::index::Index>>;

type Result<T> = std::result::Result<T, Error>;

#[derive(Debug, thiserror::Error)]
enum Error {
    #[error("IO")]
    IO(#[from] std::io::Error),

    #[error("Index: {0}")]
    Index(#[from] index::Error),

    #[error("Json: {0}")]
    Json(#[from] serde_json::Error),

    #[error("SQL: {0}")]
    Sql(#[from] sqlx::Error),

    #[error("User does not own the crate")]
    NotOwnedByUser,
}

impl IntoResponse for Error {
    fn into_response(self) -> Response {
        let status = StatusCode::BAD_REQUEST;

        let body = Json(serde_json::json!({
            "error": status.canonical_reason(),
            "reason": self.to_string(),
        }));
        let mut response = body.into_response();
        *response.status_mut() = status;

        response
    }
}

/// Start here. Returns an [`axum::Router`] configured with the routes
/// that make up the API.
///
/// For `cargo` to work with the API, these should most likely be
/// mounted at `/api/v1`. Several pieces of shared state are also
/// required.
///
/// ```
/// let app = axum::Router::new()
///     .nest("/api/v1", api::routes())
///     .layer(AddExtensionLayer::new(...));
/// ```
///
/// See also [`root_routes`].
pub fn routes() -> axum::Router {
    axum::Router::new()
        .route("/crates/:crate_name/:version/download", get(download))
        .route("/crates/new", put(publish))
        .route("/crates/:crate_name/:version/yank", delete(yank))
        .route("/crates/:crate_name/:version/unyank", put(unyank))
        .route(
            "/crates/:crate_name/owners",
            get(owners_list).put(owners_put).delete(owners_remove),
        )
        .route("/crates", get(search))
}

/// Additional routes that should be mounted at `/`.
pub fn root_routes() -> axum::Router {
    axum::Router::new().route("/me", get(me))
}

// Download

async fn download(
    Path((crate_name, version)): Path<(String, String)>,
    storage: Extension<Arc<Storage>>,
) -> Result<impl IntoResponse> {
    let path = storage
        .path
        .join(format!("{}-{}.crate", crate_name, version));
    let file = tokio::fs::File::open(path).await?;
    let stream = ReaderStream::new(file);
    let body = StreamBody::new(stream);
    Ok(body)
}

// Publish

#[derive(Deserialize)]
#[allow(unused)] // This is the public API.
struct PublishCrateBody {
    pub name: String,
    pub vers: String,
    pub deps: Vec<Dependency>,
    pub features: HashMap<String, Vec<String>>,
    pub authors: Vec<String>,
    pub description: Option<String>,
    pub homepage: Option<String>,
    pub readme: Option<String>,
    pub readme_file: Option<String>,
    pub keywords: Vec<String>,
    pub categories: Vec<String>,
    pub license: Option<String>,
    pub license_file: Option<String>,
    pub repository: Option<String>,
    #[serde(default)]
    pub badges: HashMap<String, Badge>,
    pub links: Option<String>,
}

#[derive(Deserialize)]
struct Dependency {
    /// Name of the dependency.
    ///
    /// If the dependency is renamed from the original package name,
    /// this is the original name. The new package name is stored in
    /// the `explicit_name_in_toml` field.
    pub name: String,

    /// The semver requirement for this dependency.
    pub version_req: String,

    /// Array of features (as strings) enabled for this dependency.
    pub features: Vec<String>,

    /// Boolean of whether or not this is an optional dependency.
    pub optional: bool,

    /// Boolean of whether or not default features are enabled.
    pub default_features: bool,

    /// The target platform for the dependency.
    /// null if not a target dependency.
    /// Otherwise, a string such as "cfg(windows)".
    pub target: Option<String>,

    /// The dependency kind.
    /// "dev", "build", or "normal".
    pub kind: DependencyKind,

    /// The URL of the index of the registry where this dependency is
    /// from as a string. If not specified or null, it is assumed the
    /// dependency is in the current registry.
    pub registry: Option<String>,

    /// If the dependency is renamed, this is a string of the new
    /// package name. If not specified or null, this dependency is not
    /// renamed.
    pub explicit_name_in_toml: Option<String>,
}

impl From<Dependency> for index::Dependency {
    fn from(dep: Dependency) -> Self {
        index::Dependency {
            name: dep.name,
            req: dep.version_req,
            features: dep.features,
            optional: dep.optional,
            default_features: dep.default_features,
            target: dep.target,
            kind: dep.kind.into(),
            registry: dep.registry,
            package: dep.explicit_name_in_toml,
        }
    }
}

#[derive(Deserialize)]
#[serde(rename_all = "snake_case")]
enum DependencyKind {
    Dev,
    Build,
    Normal,
}

impl From<DependencyKind> for index::DependencyKind {
    fn from(kind: DependencyKind) -> Self {
        match kind {
            DependencyKind::Dev => index::DependencyKind::Dev,
            DependencyKind::Build => index::DependencyKind::Build,
            DependencyKind::Normal => index::DependencyKind::Normal,
        }
    }
}

#[derive(Deserialize)]
#[serde(transparent)]
struct Badge(HashMap<String, String>);

#[derive(Serialize)]
struct PublishResponse {
    invalid_categories: Vec<String>,
    invalid_badges: Vec<String>,
    other: Vec<String>,
}

async fn publish(
    user: User,
    body: axum::extract::BodyStream, // Request<Body>,
    database: Extension<Arc<Database>>,
    index: Extension<Index>,
    storage: Extension<Arc<Storage>>,
) -> Result<Json<PublishResponse>> {
    // Read crate metadata.
    let mut stream =
        StreamReader::new(body.map_err(|err| std::io::Error::new(std::io::ErrorKind::Other, err)));
    let metadata_len = stream.read_u32_le().await? as usize;
    let mut metadata_buffer = vec![0; metadata_len];
    stream.read_exact(&mut metadata_buffer).await?;
    let metadata: PublishCrateBody = serde_json::from_slice(&metadata_buffer)?;

    // Either check that the user is an owner of the crate, or insert
    // a new crate into the database.
    let crate_id = {
        let mut trans = database.pool.begin().await?;

        let row = sqlx::query!("select crate_id from crate where name = $1", metadata.name)
            .fetch_optional(&mut trans)
            .await?;

        let crate_id = match row {
            // The crate exists; ensure that the user is an owner.
            Some(row) => {
                user_owns_crate(&user, &metadata.name, &database.pool).await?;
                row.crate_id
            }
            // The crate does not exist; create it and make the user
            // an owner.
            None => {
                let crate_id = sqlx::query!(
                    "insert into crate (name) values ($1) returning crate_id",
                    metadata.name
                )
                .fetch_one(&mut trans)
                .await?
                .crate_id;

                sqlx::query!(
                    "insert into crate_owner (crate_id, account_id) values ($1, $2)",
                    crate_id,
                    user.account_id as i32
                )
                .execute(&mut trans)
                .await?;

                crate_id
            }
        };

        trans.commit().await?;

        crate_id
    };

    // Read .crate file.
    let crate_len = stream.read_u32_le().await? as usize;
    let mut crate_buffer = vec![0; crate_len];
    stream.read_exact(&mut crate_buffer).await?;
    let cksum = {
        use sha2::Digest;
        let digest = sha2::Sha256::digest(&crate_buffer);
        format!("{:x}", digest)
    };

    let crate_path = storage
        .path
        .join(format!("{}-{}.crate", metadata.name, metadata.vers));
    info!(?crate_path, "opening crate file");
    let mut file = tokio::fs::OpenOptions::new()
        .write(true)
        .create_new(true)
        .open(&crate_path)
        .await?;
    info!(?crate_path, "writing crate file");
    file.write_all(&crate_buffer).await?;

    sqlx::query!(
        "update crate set description = $1 where crate_id = $2",
        metadata.description,
        crate_id
    )
    .execute(&database.pool)
    .await?;

    sqlx::query!(
        "insert into crate_version (crate_id, version) values ($1, $2)",
        crate_id,
        metadata.vers
    )
    .execute(&database.pool)
    .await?;

    let deps = metadata
        .deps
        .into_iter()
        .map(index::Dependency::from)
        .collect();
    let crate_info = index::Crate {
        name: metadata.name,
        vers: metadata.vers,
        deps,
        cksum,
        features: HashMap::new(),
        yanked: false,
        links: None,
    };

    tokio::task::block_in_place(|| {
        info!("updating index");
        let index = index.lock().unwrap();
        index.add_crate(crate_info)?;
        Result::Ok(())
    })?;

    let response = PublishResponse {
        invalid_categories: vec![],
        invalid_badges: vec![],
        other: vec![],
    };
    Ok(Json(response))
}

// Yank

#[derive(Serialize)]
struct YankResponse {
    /// Indicates the delete succeeded, always true.
    ok: bool,
}

async fn yank(
    user: User,
    Path((crate_name, version)): Path<(String, String)>,
    database: Extension<Arc<Database>>,
    index: Extension<Index>,
) -> Result<Json<YankResponse>> {
    user_owns_crate(&user, &crate_name, &database.pool).await?;

    tokio::task::block_in_place(|| {
        let index = index.lock().unwrap();
        index.yank(&crate_name, &version, true)?;
        Result::Ok(())
    })?;

    let response = YankResponse { ok: true };
    Ok(Json(response))
}

// Unyank

async fn unyank(
    user: User,
    Path((crate_name, version)): Path<(String, String)>,
    database: Extension<Arc<Database>>,
    index: Extension<Index>,
) -> Result<Json<YankResponse>> {
    user_owns_crate(&user, &crate_name, &database.pool).await?;

    tokio::task::block_in_place(|| {
        let index = index.lock().unwrap();
        index.yank(&crate_name, &version, false)?;
        Result::Ok(())
    })?;

    let response = YankResponse { ok: true };
    Ok(Json(response))
}

// Owners: List

#[derive(Serialize)]
struct OwnersListResponse {
    users: Vec<Owner>,
}

#[derive(Serialize)]
struct Owner {
    id: u32,
    login: String,
    name: Option<String>,
}

async fn owners_list(
    user: User,
    Path(crate_name): Path<String>,
    database: Extension<Arc<Database>>,
) -> Result<Json<OwnersListResponse>> {
    user_owns_crate(&user, &crate_name, &database.pool).await?;

    let owners = sqlx::query!("select account_id, username from crate join crate_owner using (crate_id) join account using (account_id) where crate.name = $1", crate_name).fetch_all(&database.pool).await?;
    let users = owners
        .into_iter()
        .map(|owner| Owner {
            id: owner.account_id as u32,
            login: owner.username,
            name: None,
        })
        .collect();

    let response = OwnersListResponse { users };
    Ok(Json(response))
}

// Owners: Add

#[derive(Deserialize)]
struct OwnersPutBody {
    /// Array of `login` strings of owners to add.
    users: Vec<String>,
}

#[derive(Serialize)]
struct OwnersPutResponse {
    ok: bool,
    msg: &'static str,
}

async fn owners_put(
    user: User,
    Path(crate_name): Path<String>,
    body: Json<OwnersPutBody>,
    database: Extension<Arc<Database>>,
) -> Result<Json<OwnersPutResponse>> {
    user_owns_crate(&user, &crate_name, &database.pool).await?;

    let mut trans = database.pool.begin().await?;
    let crate_id = sqlx::query!("select crate_id from crate where name = $1", crate_name)
        .fetch_one(&mut trans)
        .await?
        .crate_id;
    for user in &body.users {
        let account_id = sqlx::query!("select account_id from account where username = $1", user)
            .fetch_one(&mut trans)
            .await?
            .account_id;
        sqlx::query!(
            "insert into crate_owner (crate_id, account_id) values ($1, $2)",
            crate_id,
            account_id,
        )
        .execute(&mut trans)
        .await?;
    }
    trans.commit().await?;

    let response = OwnersPutResponse {
        ok: true,
        msg: "success",
    };
    Ok(Json(response))
}

// Owners: Remove

#[derive(Deserialize)]
struct OwnersDeleteBody {
    /// Array of `login` strings of owners to remove.
    users: Vec<String>,
}

#[derive(Serialize)]
struct OwnersDeleteResponse {
    ok: bool,
    msg: &'static str,
}

async fn owners_remove(
    user: User,
    Path(crate_name): Path<String>,
    body: Json<OwnersDeleteBody>,
    database: Extension<Arc<Database>>,
) -> Result<Json<OwnersDeleteResponse>> {
    user_owns_crate(&user, &crate_name, &database.pool).await?;

    let mut trans = database.pool.begin().await?;
    let crate_id = sqlx::query!("select crate_id from crate where name = $1", crate_name)
        .fetch_one(&mut trans)
        .await?
        .crate_id;
    for user in &body.users {
        sqlx::query!("delete from crate_owner where crate_id = $1 and account_id = (select account_id from account where username = $2)", crate_id, user).execute(&mut trans).await?;
    }
    trans.commit().await?;

    let response = OwnersDeleteResponse {
        ok: true,
        msg: "success",
    };
    Ok(Json(response))
}

// Search

#[derive(Deserialize)]
struct SearchQuery {
    q: Option<String>,
    per_page: Option<i64>,
}

#[derive(Serialize)]
struct SearchResponse {
    /// Array of results.
    crates: Vec<SearchResultCrate>,
    meta: SearchResponseMeta,
}

#[derive(Serialize)]
struct SearchResultCrate {
    /// Name of the crate.
    name: String,
    /// The highest version available.
    // TODO: semver
    max_version: String,
    /// Textual description of the crate.
    description: String,
}

#[derive(Serialize)]
struct SearchResponseMeta {
    /// Total number of results available on the server.
    total: usize,
}

async fn search(
    Query(SearchQuery { q, per_page }): Query<SearchQuery>,
    database: Extension<Arc<Database>>,
) -> Result<Json<SearchResponse>> {
    let crates = sqlx::query!("select name, max(version) as max_version, description from crate left join crate_version using (crate_id) where name ~ $1 group by crate_id limit $2", q.unwrap_or_default(), per_page)
        .fetch_all(&database.pool)
        .await?;
    let crates = crates
        .into_iter()
        .map(|row| SearchResultCrate {
            name: row.name,
            max_version: row.max_version.unwrap_or_else(|| "0.0.0".into()),
            description: row.description.unwrap_or_default(),
        })
        .collect();

    let response = SearchResponse {
        crates,
        meta: SearchResponseMeta { total: 1 },
    };
    Ok(Json(response))
}

// Login

// User info

#[derive(Serialize)]
struct LoggedInUser {
    username: String,
}

async fn me(user: User, database: Extension<Arc<Database>>) -> Result<Json<LoggedInUser>> {
    let row = sqlx::query!(
        "select username from account where account_id = $1",
        user.account_id as i32
    )
    .fetch_one(&database.pool)
    .await?;

    let response = LoggedInUser {
        username: row.username,
    };
    Ok(Json(response))
}

// Helpers

/// Check whether the user is an owner of the given crate.
async fn user_owns_crate(user: &User, crate_name: &str, pool: &sqlx::PgPool) -> Result<()> {
    let row = sqlx::query!(
        "select 1 as x from crate join crate_owner using (crate_id) where crate.name = $1 and crate_owner.account_id = $2",
        crate_name, user.account_id as i32
    ).fetch_optional(pool).await?;

    match row {
        Some(_) => Ok(()),
        None => Err(Error::NotOwnedByUser),
    }
}
